@@ -45,7 +45,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype Slider
     \inherits Control
-    \instantiates QQuickSlider
+//!     \instantiates QQuickSlider
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-input
@@ -272,8 +272,15 @@ QQuickSlider::QQuickSlider(QQuickItem *parent)
     : QQuickControl(*(new QQuickSliderPrivate), parent)
 {
     setActiveFocusOnTab(true);
+#ifdef Q_OS_MACOS
+    setFocusPolicy(Qt::TabFocus);
+#else
     setFocusPolicy(Qt::StrongFocus);
+#endif
     setAcceptedMouseButtons(Qt::LeftButton);
+#if QT_CONFIG(quicktemplates2_multitouch)
+    setAcceptTouchEvents(true);
+#endif
 #if QT_CONFIG(cursor)
     setCursor(Qt::ArrowCursor);
 #endif
@@ -579,7 +586,7 @@ void QQuickSlider::setHandle(QQuickItem *handle)
     const qreal oldImplicitHandleHeight = implicitHandleHeight();
 
     d->removeImplicitSizeListener(d->handle);
-    delete d->handle;
+    QQuickControlPrivate::hideOldItem(d->handle);
     d->handle = handle;
 
     if (handle) {
@@ -790,7 +797,7 @@ void QQuickSlider::mousePressEvent(QMouseEvent *event)
 {
     Q_D(QQuickSlider);
     QQuickControl::mousePressEvent(event);
-    d->handleMove(event->localPos());
+    d->handleMove(event->position());
     setKeepMouseGrab(true);
 }
 
@@ -800,26 +807,26 @@ void QQuickSlider::touchEvent(QTouchEvent *event)
     Q_D(QQuickSlider);
     switch (event->type()) {
     case QEvent::TouchUpdate:
-        for (const QTouchEvent::TouchPoint &point : event->touchPoints()) {
+        for (const QTouchEvent::TouchPoint &point : event->points()) {
             if (!d->acceptTouch(point))
                 continue;
 
             switch (point.state()) {
-            case Qt::TouchPointPressed:
-                d->handlePress(point.pos());
+            case QEventPoint::Pressed:
+                d->handlePress(point.position());
                 break;
-            case Qt::TouchPointMoved:
+            case QEventPoint::Updated:
                 if (!keepTouchGrab()) {
                     if (d->orientation == Qt::Horizontal)
-                        setKeepTouchGrab(QQuickWindowPrivate::dragOverThreshold(point.pos().x() - d->pressPoint.x(), Qt::XAxis, &point, qRound(d->touchDragThreshold)));
+                        setKeepTouchGrab(QQuickWindowPrivate::dragOverThreshold(point.position().x() - d->pressPoint.x(), Qt::XAxis, &point, qRound(d->touchDragThreshold)));
                     else
-                        setKeepTouchGrab(QQuickWindowPrivate::dragOverThreshold(point.pos().y() - d->pressPoint.y(), Qt::YAxis, &point, qRound(d->touchDragThreshold)));
+                        setKeepTouchGrab(QQuickWindowPrivate::dragOverThreshold(point.position().y() - d->pressPoint.y(), Qt::YAxis, &point, qRound(d->touchDragThreshold)));
                 }
                 if (keepTouchGrab())
-                    d->handleMove(point.pos());
+                    d->handleMove(point.position());
                 break;
-            case Qt::TouchPointReleased:
-                d->handleRelease(point.pos());
+            case QEventPoint::Released:
+                d->handleRelease(point.position());
                 break;
             default:
                 break;
@@ -848,7 +855,6 @@ void QQuickSlider::wheelEvent(QWheelEvent *event)
         const bool wasMoved = !qFuzzyCompare(d->value, oldValue);
         if (wasMoved)
             emit moved();
-        event->setAccepted(wasMoved);
     }
 }
 #endif
